@@ -69,7 +69,7 @@ sset <- function(x, ...){
 }
 #' @export
 sset.default <- function(x, i, ...){
-  if (cpp_is_simple_vec(x) && n_dots(...) == 0){
+  if (.Call(`_cheapr_cpp_is_simple_vec`, x) && n_dots(...) == 0L){
     .Call(`_cheapr_cpp_sset`, x, if (missing(i)) seq_len(vector_length(x)) else i, TRUE)
   } else {
     if (!missing(i) && is.logical(i)){
