@@ -130,6 +130,19 @@
 #define YIELD(n) (Rf_unprotect(n))
 #endif
 
+
+inline void *safe_memmove(void *dst, const void *src, size_t n){
+  return n ? memmove(dst, src, n) : dst;
+}
+
+inline void *safe_memcpy(void *dst, const void *src, size_t n){
+  return n ? memcpy(dst, src, n) : dst;
+}
+
+inline void *safe_memset(void *dst, int val, size_t n){
+  return n ? memset(dst, val, n) : dst;
+}
+
 template<typename T>
 inline constexpr bool between(T x, T lo, T hi) {
   return x >= lo && x <= hi;
@@ -144,7 +157,6 @@ inline constexpr bool is_integerable(T x){
 int num_cores();
 SEXP cpp_which_(SEXP x, bool invert);
 SEXP cpp_missing_row(SEXP x, double threshold, bool threshold_is_prop);
-int int_div(int x, int y);
 SEXP xlen_to_r(R_xlen_t x);
 R_xlen_t vec_length(SEXP x);
 SEXP r_address(SEXP x);
